@@ -117,7 +117,7 @@ class ControllerNode(object):
         free_workers = []
         free_local_workers = []
 
-        for worker_id, worker in self.worker_map.items():
+        for worker_id, worker in self.worker_map.copy().items():
             # ignore downloader workers
             if worker.get('workertype') in ('download', 'moveparquet'):
                 continue
@@ -516,13 +516,16 @@ class ControllerNode(object):
         return 'hai!'
 
     def get_info(self):
-        data = {'msg_count_in': self.msg_count_in, 'node': self.node_name,
-                'workers': self.worker_map,
+        data = {'msg_count_in': self.msg_count_in,
+                'node': self.node_name,
+                'workers': {k.decode(): v for k, v in self.worker_map.items()},
                 'worker_out_messages': [(k, len(v)) for k, v in self.worker_out_messages.items()],
-                'last_heartbeat': self.last_heartbeat, 'address': self.address.decode(),
+                'last_heartbeat': self.last_heartbeat,
+                'address': self.address.decode(),
                 'others': {k.decode(): v for k, v in self.others.items()},
                 'rpc_results_len': len(self.rpc_results),
-                'uptime': int(time.time() - self.start_time), 'start_time': self.start_time
+                'uptime': int(time.time() - self.start_time),
+                'start_time': self.start_time
                 }
         return data
 
@@ -536,7 +539,7 @@ class ControllerNode(object):
 
     def free_dead_workers(self):
         now = time.time()
-        for worker_id, worker in self.worker_map.items():
+        for worker_id, worker in self.worker_map.copy().items():
             if (now - worker.get('last_seen', now)) > DEAD_WORKER_TIMEOUT:
                 self.remove_worker(worker_id)
 
